import os
from typing import List
import datetime

class DeltaValueError(Exception):
    """Custom exception for invalid delta format."""
    pass

def get_major_tz_from_env() -> List[str]:
    try:
        major_time_zones = os.environ.get("MAJOR_TIMEZONES", None)
        return list(major_time_zones[1:len(major_time_zones) - 1].split(','))
    except:
        return []

def callDelta(time: datetime.datetime, delta: str) -> datetime.datetime:
    if 'hr' in delta:
        hours = int(delta.replace('hr', ''))
        time += datetime.timedelta(hours=hours)
    elif 'min' in delta:
        minutes = int(delta.replace('min', ''))
        time += datetime.timedelta(minutes=minutes)
    elif 'day' in delta:
        days = int(delta.replace('day', ''))
        time += datetime.timedelta(days=days)
    else:
        raise DeltaValueError("Invalid time format. Use +2hr, -30min, -1day, etc.")
    return time


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='worldtimebuddy',
    version='0.1.3',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Click',
        'pytz',
    ],
    entry_points='''
        [console_scripts]
        worldtimebuddy=worldtimebuddy.cli:cli
    ''',
    description='A command-line tool to display current time for different timezones',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Sarvesh Dubey',
    author_email='dubeysarvesh5525@gmail.com',
    url='https://github.com/dubesar/worldtimebuddy',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
)


timezone_codes = {
    'IST': 'Asia/Kolkata',
    'PST': 'US/Pacific',
    'EST': 'US/Eastern',
    'CST': 'US/Central',
    'MST': 'US/Mountain',
    'GMT': 'Europe/London',
    'CET': 'Europe/Paris',
    'EET': 'Europe/Athens',
    'JST': 'Asia/Tokyo',
    'AEST': 'Australia/Sydney',
    'UTC': 'UTC'
}


import datetime
import pytz
import click

from worldtimebuddy.utils import get_major_tz_from_env, callDelta, DeltaValueError
from worldtimebuddy.constants import timezone_codes

MAJOR_TIMEZONES = get_major_tz_from_env() or ['UTC', 'US/Pacific', 'Asia/Kolkata']

@click.command()
@click.option('--format', default='%Y-%m-%d %H:%M:%S', help='DateTime format string')
@click.option('--major', is_flag=True, help='Show only major timezones')
@click.option('--timezone', '-tz', help='Show time for a specific timezone')
@click.option('--list', 'list_timezones', is_flag=True, help='List all available timezones')
@click.option('--delta', help='Time to add (e.g., +2hr, -30min, -1day)')
def cli(format, major, timezone, list_timezones, delta):
    """
    Display current time for all timezones, major timezones, or a specific timezone.
    """
    if list_timezones:
        for tz in pytz.all_timezones:
            click.echo(tz)
        return

    now = datetime.datetime.now(pytz.utc)
    
    if timezone:
        # Map short timezone code to full name if applicable
        timezone = timezone_codes.get(timezone, timezone)
        
        try:
            tz = pytz.timezone(timezone)
            time = now.astimezone(tz)
            
            if delta:
                try:
                    time = callDelta(time, delta)
                except DeltaValueError as e:
                    click.echo(f"DeltaValueError: {e}")
                    return

            click.echo(f"{timezone:<30} {time.strftime(format)}")
        except pytz.exceptions.UnknownTimeZoneError:
            click.echo(f"Error: Unknown timezone '{timezone}'")
            click.echo("Use 'gettime --list' to see all available timezones")
    elif major:
        for tz in MAJOR_TIMEZONES:
            timezone = pytz.timezone(tz)
            time = now.astimezone(timezone)
            if tz == 'US/Pacific':
                pdt_pst = 'PDT' if time.tzinfo._dst else 'PST'
                click.echo(f"{pdt_pst:<30} {time.strftime(format)}")
            else:
                click.echo(f"{tz:<30} {time.strftime(format)}")
    else:
        for tz in pytz.all_timezones:
            timezone = pytz.timezone(tz)
            time = now.astimezone(timezone)
            click.echo(f"{tz:<30} {time.strftime(format)}")

if __name__ == '__main__':
    cli()